from decimal import Decimal

import numpy as np

from .datatypes import CameraCenters


def load_file(path) -> CameraCenters:
    """
    loads the frame_trajectory.txt file generated by openvslam and returns a CameraCenters object.

    :param path: path to list
    :return: list of 3d points (np.arrays)
    """
    xx = []
    yy = []
    zz = []
    quats = []

    with open(path) as f:
        frame_tra_all = f.read().splitlines()

    for frame_tra in frame_tra_all:
        frame_tra_list = frame_tra.split(' ')
        quats.append((Decimal(frame_tra_list[7]),
                      Decimal(frame_tra_list[4]),
                      Decimal(frame_tra_list[5]),
                      Decimal(frame_tra_list[6])))

        location_x = float(frame_tra_list[1])
        location_y = float(frame_tra_list[2])
        location_z = float(frame_tra_list[3])
        xx.append(location_x)
        yy.append(location_y)
        zz.append(location_z)

    n = len(xx)
    res = CameraCenters([np.array((xx[i], yy[i], zz[i])) for i in range(n)])
    res.orientations = quats
    return res
